@@ -539,6 +539,8 @@ update_attr_iterator (ItemizeState *state)
     state->attr_end = state->end;
 
   old_lang = state->lang;
+  if (state->font_desc)
+    pango_font_description_free (state->font_desc);
   state->font_desc = pango_font_description_copy_static (state->context->font_desc);
   pango_attr_iterator_get_font (state->attr_iter, state->font_desc,
 				&state->lang, &state->extra_attrs);
@@ -609,6 +611,7 @@ itemize_state_init (ItemizeState      *state,
       state->free_attr_iter = TRUE;
     }
 
+  state->font_desc = NULL;
   state->lang = NULL;
 
   advance_attr_iterator_to (state->attr_iter, start_index);
@@ -869,8 +872,15 @@ compute_derived_language (PangoLanguage *lang,
   else
     {
       derived_lang = pango_script_get_sample_language (script);
+      /* If we don't find a sample language for the script, we
+       * use a language tag that shouldn't actually be used
+       * anywhere. This keeps fontconfig (for the PangoFc*
+       * backend) from using the language tag to affect the
+       * sort order. I don't have a reference for 'xx' being
+       * safe here, though Keith Packard claims it is.
+       */
       if (!derived_lang)
-	derived_lang = lang;
+	derived_lang = pango_language_from_string ("xx");
     }
 
   return derived_lang;
@@ -941,6 +951,9 @@ itemize_state_process_run (ItemizeState *state)
 
   itemize_state_update_for_new_run (state);
 
+  /* We should never get an empty run */
+  g_assert (state->run_end != state->run_start);
+
   for (p = state->run_start;
        p < state->run_end;
        p = g_utf8_next_char (p))
