@@ -767,8 +767,11 @@ big_parse_func      (MarkupData            *md,
   CHECK_NO_ATTRS("big");
 
   /* Grow text one level */
-  tag->scale_level_delta += 1;
-  tag->scale_level += 1;
+  if (tag)
+    {
+      tag->scale_level_delta += 1;
+      tag->scale_level += 1;
+    }
 
   return TRUE;
 }
@@ -816,7 +819,8 @@ parse_absolute_size (OpenTag               *tag,
    */
   factor = scale_factor (level, 1.0);
   add_attribute (tag, pango_attr_scale_new (factor));
-  open_tag_set_absolute_font_scale (tag, factor);
+  if (tag)
+    open_tag_set_absolute_font_scale (tag, factor);
   
   return TRUE;
 }
@@ -950,7 +954,8 @@ span_parse_func     (MarkupData            *md,
       if (parsed)
         {
           add_attribute (tag, pango_attr_font_desc_new (parsed));
-          open_tag_set_absolute_font_size (tag, pango_font_description_get_size (parsed));
+	  if (tag)
+	    open_tag_set_absolute_font_size (tag, pango_font_description_get_size (parsed));
           pango_font_description_free (parsed);
         }
     }
@@ -982,17 +987,24 @@ span_parse_func     (MarkupData            *md,
             }
 
           add_attribute (tag, pango_attr_size_new (n));
-          open_tag_set_absolute_font_size (tag, n);
+	  if (tag)
+	    open_tag_set_absolute_font_size (tag, n);
         }
       else if (strcmp (size, "smaller") == 0)
         {
-          tag->scale_level_delta -= 1;
-          tag->scale_level -= 1;
+	  if (tag)
+	    {
+	      tag->scale_level_delta -= 1;
+	      tag->scale_level -= 1;
+	    }
         }
       else if (strcmp (size, "larger") == 0)
         {
-          tag->scale_level_delta += 1;
-          tag->scale_level += 1;
+	  if (tag)
+	    {
+	      tag->scale_level_delta += 1;
+	      tag->scale_level += 1;
+	    }
         }
       else if (parse_absolute_size (tag, size))
         ; /* nothing */
@@ -1259,8 +1271,11 @@ sub_parse_func      (MarkupData            *md,
   CHECK_NO_ATTRS("sub");
 
   /* Shrink font, and set a negative rise */
-  tag->scale_level_delta -= 1;
-  tag->scale_level -= 1;
+  if (tag)
+    {
+      tag->scale_level_delta -= 1;
+      tag->scale_level -= 1;
+    }
 
   add_attribute (tag, pango_attr_rise_new (-SUPERSUB_RISE));
 
@@ -1278,8 +1293,11 @@ sup_parse_func      (MarkupData            *md,
   CHECK_NO_ATTRS("sup");
 
   /* Shrink font, and set a positive rise */
-  tag->scale_level_delta -= 1;
-  tag->scale_level -= 1;
+  if (tag)
+    {
+      tag->scale_level_delta -= 1;
+      tag->scale_level -= 1;
+    }
 
   add_attribute (tag, pango_attr_rise_new (SUPERSUB_RISE));
 
@@ -1297,8 +1315,11 @@ small_parse_func    (MarkupData            *md,
   CHECK_NO_ATTRS("small");
 
   /* Shrink text one level */
-  tag->scale_level_delta -= 1;
-  tag->scale_level -= 1;
+  if (tag)
+    {
+      tag->scale_level_delta -= 1;
+      tag->scale_level -= 1;
+    }
 
   return TRUE;
 }
